@@ -10,9 +10,10 @@
 
 
 define([
-    'base/js/namespace',
     'jquery',
+    'base/js/namespace',
     'base/js/utils',
+    'base/js/i18n',
     'base/js/keyboard',
     'services/config',
     'notebook/js/cell',
@@ -22,9 +23,11 @@ define([
     'codemirror/lib/codemirror',
     'codemirror/mode/python/python',
     'notebook/js/codemirror-ipython'
-], function(IPython,
+], function(
     $,
+    IPython,
     utils,
+    i18n,
     keyboard,
     configmod,
     cell,
@@ -49,17 +52,29 @@ define([
      * @private
      */
     CodeMirror.commands.delSpaceToPrevTabStop = function(cm){
-        var from = cm.getCursor(true), to = cm.getCursor(false), sel = !posEq(from, to);
-        if (!posEq(from, to)) { cm.replaceRange("", from, to); return; }
-        var cur = cm.getCursor(), line = cm.getLine(cur.line);
-        var tabsize = cm.getOption('tabSize');
-        var chToPrevTabStop = cur.ch-(Math.ceil(cur.ch/tabsize)-1)*tabsize;
-        from = {ch:cur.ch-chToPrevTabStop,line:cur.line};
-        var select = cm.getRange(from,cur);
-        if( select.match(/^\ +$/) !== null){
-            cm.replaceRange("",from,cur);
-        } else {
-            cm.deleteH(-1,"char");
+        var tabSize = cm.getOption('tabSize');
+        var ranges = cm.listSelections(); // handle multicursor
+        for (var i = ranges.length - 1; i >= 0; i--) { // iterate reverse so any deletions don't overlap
+            var head = ranges[i].head;
+            var anchor = ranges[i].anchor;
+            var sel = !posEq(head, anchor);
+            if (sel) {
+                // range is selection
+                cm.replaceRange("", anchor, head);
+            } else {
+                // range is cursor
+                var line = cm.getLine(head.line).substring(0, head.ch);
+                if (line.match(/^\ +$/) !== null){
+                    // delete tabs
+                    var prevTabStop = (Math.ceil(head.ch/tabSize)-1)*tabSize;
+                    var from = CodeMirror.Pos(head.line, prevTabStop)
+                    cm.replaceRange("", from, head);
+                } else {
+                    // delete normally
+                    var from = cm.findPosH(head, -1,  'char', false);
+                    cm.replaceRange("", from, head);
+                }
+            }
         }
     };
 
@@ -90,7 +105,7 @@ define([
         this.tooltip = options.tooltip;
         this.config = options.config;
         this.class_config = new configmod.ConfigWithDefaults(this.config,
-                                        CodeCell.config_defaults, 'CodeCell');
+                                        CodeCell.options_default, 'CodeCell');
 
         // create all attributed in constructor function
         // even if null for V8 VM optimisation
@@ -102,7 +117,7 @@ define([
         this.completer = null;
 
         Cell.apply(this,[{
-            config: $.extend({}, CodeCell.options_default), 
+            config: options.config, 
             keyboard_manager: options.keyboard_manager, 
             events: this.events}]);
 
@@ -117,11 +132,7 @@ define([
     CodeCell.options_default = {
         cm_config : {
             extraKeys: {
-                "Tab" :  "indentMore",
-                "Shift-Tab" : "indentLess",
                 "Backspace" : "delSpaceToPrevTabStop",
-                "Cmd-/" : "toggleComment",
-                "Ctrl-/" : "toggleComment"
             },
             mode: 'text',
             theme: 'ipython',
@@ -139,8 +150,6 @@ define([
         },
     };
 
-    CodeCell.config_defaults = CodeCell.options_default;
-
     CodeCell.msg_cells = {};
 
     CodeCell.prototype = Object.create(Cell.prototype);
@@ -155,13 +164,25 @@ define([
 
         var input = $('<div></div>').addClass('input');
         this.input = input;
+
+        var prompt_container = $('<div/>').addClass('prompt_container');
+
+        var run_this_cell = $('<div></div>').addClass('run_this_cell');
+        run_this_cell.prop('title', 'Run this cell');
+        run_this_cell.append('<i class="fa-step-forward fa"></i>');
+        run_this_cell.click(function (event) {
+            event.stopImmediatePropagation();
+            that.execute();
+        });
+
         var prompt = $('<div/>').addClass('prompt input_prompt');
+        
         var inner_cell = $('<div/>').addClass('inner_cell');
         this.celltoolbar = new celltoolbar.CellToolbar({
             cell: this, 
             notebook: this.notebook});
         inner_cell.append(this.celltoolbar.element);
-        var input_area = $('<div/>').addClass('input_area');
+        var input_area = $('<div/>').addClass('input_area').attr("aria-label", i18n.msg._("Edit code here"));
         this.code_mirror = new CodeMirror(input_area.get(0), this._options.cm_config);
         // In case of bugs that put the keyboard manager into an inconsistent state,
         // ensure KM is enabled when CodeMirror is focused:
@@ -169,31 +190,42 @@ define([
             if (that.keyboard_manager) {
                 that.keyboard_manager.enable();
             }
+
+            that.code_mirror.setOption('readOnly', !that.is_editable());
         });
         this.code_mirror.on('keydown', $.proxy(this.handle_keyevent,this));
         $(this.code_mirror.getInputField()).attr("spellcheck", "false");
         inner_cell.append(input_area);
-        input.append(prompt).append(inner_cell);
+        prompt_container.append(prompt).append(run_this_cell);
+        input.append(prompt_container).append(inner_cell);
 
         var output = $('<div></div>');
         cell.append(input).append(output);
         this.element = cell;
         this.output_area = new outputarea.OutputArea({
-            selector: output, 
-            prompt_area: true, 
-            events: this.events, 
-            keyboard_manager: this.keyboard_manager});
+            config: this.config,
+            selector: output,
+            prompt_area: true,
+            events: this.events,
+            keyboard_manager: this.keyboard_manager,
+        });
         this.completer = new completer.Completer(this, this.events);
     };
 
     /** @method bind_events */
     CodeCell.prototype.bind_events = function () {
-        Cell.prototype.bind_events.apply(this);
+        Cell.prototype.bind_events.apply(this, arguments);
         var that = this;
 
         this.element.focusout(
             function() { that.auto_highlight(); }
         );
+
+        this.events.on('kernel_restarting.Kernel', function() {
+            if (that.input_prompt_number === '*') {
+              that.set_input_prompt();
+            }
+        });
     };
 
 
@@ -297,24 +329,30 @@ define([
      * @method execute
      */
     CodeCell.prototype.execute = function (stop_on_error) {
-        if (!this.kernel || !this.kernel.is_connected()) {
-            console.log("Can't execute, kernel is not connected.");
+        if (!this.kernel) {
+            console.log(i18n.msg._("Can't execute cell since kernel is not set."));
             return;
         }
 
-        this.output_area.clear_output(false, true);
-
         if (stop_on_error === undefined) {
-            stop_on_error = true;
+            if (this.metadata !== undefined && 
+                    this.metadata.tags !== undefined) {
+                if (this.metadata.tags.indexOf('raises-exception') !== -1) {
+                    stop_on_error = false;
+                } else {
+                    stop_on_error = true;
+                }
+            } else {
+               stop_on_error = true;
+            }
         }
 
+        this.clear_output(false, true);
         var old_msg_id = this.last_msg_id;
-
         if (old_msg_id) {
             this.kernel.clear_callbacks_for_msg(old_msg_id);
-            if (old_msg_id) {
-                delete CodeCell.msg_cells[old_msg_id];
-            }
+            delete CodeCell.msg_cells[old_msg_id];
+            this.last_msg_id = null;
         }
         if (this.get_text().trim().length === 0) {
             // nothing to do
@@ -330,6 +368,14 @@ define([
         CodeCell.msg_cells[this.last_msg_id] = this;
         this.render();
         this.events.trigger('execute.CodeCell', {cell: this});
+        var that = this;
+        function handleFinished(evt, data) {
+            if (that.kernel.id === data.kernel.id && that.last_msg_id === data.msg_id) {
+                    that.events.trigger('finished_execute.CodeCell', {cell: that});
+                that.events.off('finished_iopub.Kernel', handleFinished);
+              }
+        }
+        this.events.on('finished_iopub.Kernel', handleFinished);
     };
     
     /**
@@ -339,6 +385,7 @@ define([
     CodeCell.prototype.get_callbacks = function () {
         var that = this;
         return {
+            clear_on_done: false,
             shell : {
                 reply : $.proxy(this._handle_execute_reply, this),
                 payload : {
@@ -348,13 +395,15 @@ define([
             },
             iopub : {
                 output : function() { 
+                    that.events.trigger('set_dirty.Notebook', {value: true});
                     that.output_area.handle_output.apply(that.output_area, arguments);
                 }, 
                 clear_output : function() { 
+                    that.events.trigger('set_dirty.Notebook', {value: true});
                     that.output_area.handle_clear_output.apply(that.output_area, arguments);
                 }, 
             },
-            input : $.proxy(this._handle_input_request, this)
+            input : $.proxy(this._handle_input_request, this),
         };
     };
     
@@ -377,7 +426,12 @@ define([
      * @private
      */
     CodeCell.prototype._handle_set_next_input = function (payload) {
-        var data = {'cell': this, 'text': payload.text, replace: payload.replace};
+        var data = {
+            cell: this,
+            text: payload.text,
+            replace: payload.replace,
+            clear_output: payload.clear_output,
+        };
         this.events.trigger('set_next_input.Notebook', data);
     };
 
@@ -393,7 +447,7 @@ define([
     // Basic cell manipulation.
 
     CodeCell.prototype.select = function () {
-        var cont = Cell.prototype.select.apply(this);
+        var cont = Cell.prototype.select.apply(this, arguments);
         if (cont) {
             this.code_mirror.refresh();
             this.auto_highlight();
@@ -402,7 +456,7 @@ define([
     };
 
     CodeCell.prototype.render = function () {
-        var cont = Cell.prototype.render.apply(this);
+        var cont = Cell.prototype.render.apply(this, arguments);
         // Always execute, even if we are already in the rendered state
         return cont;
     };
@@ -447,7 +501,7 @@ define([
         } else {
             ns = encodeURIComponent(prompt_value);
         }
-        return 'In&nbsp;[' + ns + ']:';
+        return '<bdi>'+i18n.msg._('In')+'</bdi>&nbsp;[' + ns + ']:';
     };
 
     CodeCell.input_prompt_continuation = function (prompt_value, lines_number) {
@@ -468,8 +522,10 @@ define([
         }
         this.input_prompt_number = number;
         var prompt_html = CodeCell.input_prompt_function(this.input_prompt_number, nline);
+
         // This HTML call is okay because the user contents are escaped.
         this.element.find('div.input_prompt').html(prompt_html);
+        this.events.trigger('set_dirty.Notebook', {value: true});
     };
 
 
@@ -488,8 +544,9 @@ define([
     };
 
 
-    CodeCell.prototype.clear_output = function (wait) {
-        this.output_area.clear_output(wait);
+    CodeCell.prototype.clear_output = function (wait, ignore_queue) {
+        this.events.trigger('clear_output.CodeCell', {cell: this});
+        this.output_area.clear_output(wait, ignore_queue);
         this.set_input_prompt();
     };
 
@@ -525,7 +582,11 @@ define([
         var outputs = this.output_area.toJSON();
         data.outputs = outputs;
         data.metadata.trusted = this.output_area.trusted;
-        data.metadata.collapsed = this.output_area.collapsed;
+        if (this.output_area.collapsed) {
+            data.metadata.collapsed = this.output_area.collapsed;
+        } else {
+            delete data.metadata.collapsed;
+        }
         if (this.output_area.scroll_state === 'auto') {
             delete data.metadata.scrolled;
         } else {
@@ -535,12 +596,12 @@ define([
     };
 
     /**
-     * handle cell level logic when the cursor moves away from a cell
+     * handle cell level logic when the cell is unselected
      * @method unselect
      * @return is the action being taken
      */
-    CodeCell.prototype.unselect = function (leave_selected) {
-        var cont = Cell.prototype.unselect.apply(this, [leave_selected]);
+    CodeCell.prototype.unselect = function() {
+        var cont = Cell.prototype.unselect.apply(this, arguments);
         if (cont) {
             // When a code cell is unselected, make sure that the corresponding
             // tooltip and completer to that cell is closed.
@@ -552,7 +613,7 @@ define([
         return cont;
     };
 
-    // Backwards compatability.
+    // Backwards compatibility.
     IPython.CodeCell = CodeCell;
 
     return {'CodeCell': CodeCell};
