@@ -1,10 +1,7 @@
 // Copyright (c) Jupyter Development Team.
 // Distributed under the terms of the Modified BSD License.
 
-define([
-    'jquery',
-    'bootstraptour',
-], function($, Tour) {
+define(['jquery', 'bootstraptour', 'base/js/i18n'], function($, Tour, i18n) {
     "use strict";
 
     var tour_style = "<div class='popover tour'>\n" +
@@ -27,92 +24,100 @@ define([
         this.events = events;
         this.tour_steps = [
             { 
-                title: "Welcome to the Notebook Tour",
+                title: i18n.msg._("Welcome to the Notebook Tour"),
                 placement: 'bottom',
                 orphan: true,
-                content: "You can use the left and right arrow keys to go backwards and forwards."
+                content: i18n.msg._("You can use the left and right arrow keys to go backwards and forwards.")
             }, {
                 element: "#notebook_name",
-                title: "Filename",
+                title: i18n.msg._("Filename"),
                 placement: 'bottom',
-                content: "Click here to change the filename for this notebook."
+                content: i18n.msg._("Click here to change the filename for this notebook.")
             }, {
                 element: $("#menus").parent(),
                 placement: 'bottom',
-                title: "Notebook Menubar",
-                content: "The menubar has menus for actions on the notebook, its cells, and the kernel it communicates with."
+                title: i18n.msg._("Notebook Menubar"),
+                content: i18n.msg._("The menubar has menus for actions on the notebook, its cells, and the kernel it communicates with.")
             }, {
                 element: "#maintoolbar",
                 placement: 'bottom',
-                title: "Notebook Toolbar",
-                content: "The toolbar has buttons for the most common actions. Hover your mouse over each button for more information."
+                title: i18n.msg._("Notebook Toolbar"),
+                content: i18n.msg._("The toolbar has buttons for the most common actions. Hover your mouse over each button for more information.")
             }, {
                 element: "#modal_indicator",
-                title: "Mode Indicator",
+                title: i18n.msg._("Mode Indicator"),
                 placement: 'bottom',
-                content: "The Notebook has two modes: Edit Mode and Command Mode. In this area, an indicator can appear to tell you which mode you are in.",
+                content: i18n.msg._("The Notebook has two modes: Edit Mode and Command Mode. In this area, an indicator can appear to tell you which mode you are in."),
                 onShow: function(tour) { that.command_icon_hack(); }
             }, {
                 element: "#modal_indicator",
-                title: "Command Mode",
+                title: i18n.msg._("Command Mode"),
                 placement: 'bottom',
                 onShow: function(tour) { notebook.command_mode(); that.command_icon_hack(); },
                 onNext: function(tour) { that.edit_mode(); },
-                content: "Right now you are in Command Mode, and many keyboard shortcuts are available. In this mode, no icon is displayed in the indicator area."
+                content: i18n.msg._("Right now you are in Command Mode, and many keyboard shortcuts are available. In this mode, no icon is displayed in the indicator area.")
             }, {
                 element: "#modal_indicator",
-                title: "Edit Mode",
+                title: i18n.msg._("Edit Mode"),
                 placement: 'bottom',
                 onShow: function(tour) { that.edit_mode(); },
-                content: "Pressing <code>Enter</code> or clicking in the input text area of the cell switches to Edit Mode."
+                content: i18n.msg._("Pressing <code>Enter</code> or clicking in the input text area of the cell switches to Edit Mode.")
             }, {
                 element: '.selected',
-                title: "Edit Mode",
+                title: i18n.msg._("Edit Mode"),
                 placement: 'bottom',
                 onShow: function(tour) { that.edit_mode(); },
-                content: "Notice that the border around the currently active cell changed color. Typing will insert text into the currently active cell."
+                content: i18n.msg._("Notice that the border around the currently active cell changed color. Typing will insert text into the currently active cell.")
             }, {
                 element: '.selected',
-                title: "Back to Command Mode",
+                title: i18n.msg._("Back to Command Mode"),
                 placement: 'bottom',
                 onShow: function(tour) { notebook.command_mode(); },
-                onHide: function(tour) { $('#help_menu').parent().children('a').click(); },
-                content: "Pressing <code>Esc</code> or clicking outside of the input text area takes you back to Command Mode."
+                content: i18n.msg._("Pressing <code>Esc</code> or clicking outside of the input text area takes you back to Command Mode.")
             }, {
                 element: '#keyboard_shortcuts',
-                title: "Keyboard Shortcuts",
+                title: i18n.msg._("Keyboard Shortcuts"),
                 placement: 'bottom',
-                onHide: function(tour) { $('#help_menu').parent().children('a').click(); },
-                content: "You can click here to get a list of all of the keyboard shortcuts."
+                onShow: function(tour) {
+                    /** need to add `open` and `pulse` classes in 2 calls */
+                    $('#help_menu').parent().addClass('open');
+                    $('#help_menu').parent().addClass('pulse');
+                    $('#keyboard_shortcuts').addClass('pulse');
+                  },
+                onHide: function(tour) { 
+                    $('#help_menu').parent().removeClass('open pulse');
+                    $('#keyboard_shortcuts').removeClass('pulse');
+                  },
+                content: i18n.msg._("You can click here to get a list of all of the keyboard shortcuts.")
             }, {
                 element: "#kernel_indicator_icon",
-                title: "Kernel Indicator",
+                title: i18n.msg._("Kernel Indicator"),
                 placement: 'bottom',
                 onShow: function(tour) { events.trigger('kernel_idle.Kernel');},
-                content: "This is the Kernel indicator. It looks like this when the Kernel is idle."
+                content: i18n.msg._("This is the Kernel indicator. It looks like this when the Kernel is idle.")
             }, {
                 element: "#kernel_indicator_icon",
-                title: "Kernel Indicator",
+                title: i18n.msg._("Kernel Indicator"),
                 placement: 'bottom',
                 onShow: function(tour) { events.trigger('kernel_busy.Kernel'); },
-                content: "The Kernel indicator looks like this when the Kernel is busy."
+                content: i18n.msg._("The Kernel indicator looks like this when the Kernel is busy.")
             }, {
                 element: ".fa-stop",
                 placement: 'bottom',
-                title: "Interrupting the Kernel",
+                title: i18n.msg._("Interrupting the Kernel"),
                 onHide: function(tour) { events.trigger('kernel_idle.Kernel'); },
-                content: "To cancel a computation in progress, you can click here."
+                content: i18n.msg._("To cancel a computation in progress, you can click here.")
             }, {
                 element: "#notification_kernel",
                 placement: 'bottom',
                 onShow: function(tour) { $('.fa-stop').click(); },
-                title: "Notification Area",
-                content: "Messages in response to user actions (Save, Interrupt, etc) appear here."
+                title: i18n.msg._("Notification Area"),
+                content: i18n.msg._("Messages in response to user actions (Save, Interrupt, etc.) appear here.")
             }, {
-                title: "Fin.",
+                title: i18n.msg._("End of Tour"),
                 placement: 'bottom',
                 orphan: true,
-                content: "This concludes the Jupyter Notebook User Interface Tour. Happy hacking!"
+                content: i18n.msg._("This concludes the Jupyter Notebook User Interface Tour.")
             }
         ];
 
@@ -146,7 +151,7 @@ define([
     };
 
     NotebookTour.prototype.command_icon_hack =  function() {
-        $('#modal_indicator').css('min-height', 20);
+        $('#modal_indicator').css('min-height', '18px');
     };
     
     NotebookTour.prototype.toggle_pause_play = function () { 
